@@ -1,11 +1,12 @@
 #!/usr/bin/env python
 from basetest import BaseTest, empty_feed, DummyPackageKit
-import sys, os, tempfile
+import sys, os, tempfile, imp
 from StringIO import StringIO
 import unittest
 
 sys.path.insert(0, '..')
 from zeroinstall.injector import distro, model, qdom, iface_cache, handler
+from zeroinstall.support import basedir
 
 def parse_impls(impls):
 	xml = """<?xml version="1.0" ?>
@@ -246,6 +247,10 @@ class TestDistro(BaseTest):
 		self.assertEqual('0.3.1-1', distro.try_cleanup_distro_version('0.3.1-1ubuntu0'))
 		self.assertEqual('0.3-post1-rc2', distro.try_cleanup_distro_version('0.3-post1-rc2'))
 		self.assertEqual('0.3.1-2', distro.try_cleanup_distro_version('0.3.1-r2-r3'))
+		self.assertEqual('6.17', distro.try_cleanup_distro_version('6b17'))
+		self.assertEqual('20-1', distro.try_cleanup_distro_version('b20_1'))
+		self.assertEqual('17', distro.try_cleanup_distro_version('p17'))
+		self.assertEqual(None, distro.try_cleanup_distro_version('cvs'))
 
 	def testCommand(self):
 		dpkgdir = os.path.join(os.path.dirname(__file__), 'dpkg')
@@ -262,5 +267,36 @@ class TestDistro(BaseTest):
 		impl, = icache.get_feed(master_feed.get_distro_feed()).implementations.values()
 		self.assertEqual('/bin/sh', impl.main)
 
+	def testPortable(self):
+		# Overrides all XDG_* variables
+		os.environ['ZEROINSTALL_PORTABLE_BASE'] = '/portable'
+		imp.reload(basedir)
+		self.assertEqual('/portable/config', basedir.xdg_config_home)
+		self.assertEqual('/portable/cache', basedir.xdg_cache_home)
+		self.assertEqual('/portable/data', basedir.xdg_data_home)
+
+		self.assertEqual(['/portable/config'], basedir.xdg_config_dirs)
+		self.assertEqual(['/portable/cache'], basedir.xdg_cache_dirs)
+		self.assertEqual(['/portable/data'], basedir.xdg_data_dirs)
+
+		del os.environ['ZEROINSTALL_PORTABLE_BASE']
+		os.environ['XDG_CONFIG_HOME'] = '/home/me/config'
+		os.environ['XDG_CONFIG_DIRS'] = '/system/config'
+
+		os.environ['XDG_DATA_HOME'] = '/home/me/data'
+		os.environ['XDG_DATA_DIRS'] = '/system/data' + os.pathsep + '/disto/data'
+
+		os.environ['XDG_CACHE_HOME'] = '/home/me/cache'
+		os.environ['XDG_CACHE_DIRS'] = '/system/cache'
+		imp.reload(basedir)
+
+		self.assertEqual('/home/me/config', basedir.xdg_config_home)
+		self.assertEqual('/home/me/cache', basedir.xdg_cache_home)
+		self.assertEqual('/home/me/data', basedir.xdg_data_home)
+
+		self.assertEqual(['/home/me/config', '/system/config'], basedir.xdg_config_dirs)
+		self.assertEqual(['/home/me/cache', '/system/cache'], basedir.xdg_cache_dirs)
+		self.assertEqual(['/home/me/data', '/system/data', '/disto/data'], basedir.xdg_data_dirs)
+
 if __name__ == '__main__':
 	unittest.main()
