@@ -37,7 +37,7 @@ def get_tooltip_text(mainwindow, interface, main_feed, model_column):
 	elif model_column is None:
 		return _("Click here for more options...")
 
-	impl = mainwindow.policy.implementation.get(interface, None)
+	impl = mainwindow.driver.solver.selections.get(interface, None)
 	if not impl:
 		return _("No suitable version was found. Double-click "
 			 "here to find out why.")
@@ -53,10 +53,10 @@ def get_tooltip_text(mainwindow, interface, main_feed, model_column):
 
 	assert model_column == InterfaceBrowser.DOWNLOAD_SIZE
 
-	if mainwindow.policy.get_cached(impl):
+	if impl.is_available(mainwindow.driver.config.stores):
 		return _("This version is already stored on your computer.")
 	else:
-		src = mainwindow.policy.fetcher.get_best_source(impl)
+		src = mainwindow.driver.config.fetcher.get_best_source(impl)
 		if not src:
 			return _("No downloads available!")
 		return _("Need to download %(pretty_size)s (%(size)s bytes)") % \
@@ -147,7 +147,8 @@ class InterfaceBrowser:
 	model = None
 	root = None
 	cached_icon = None
-	policy = None
+	driver = None
+	config = None
 	original_implementation = None
 	update_icons = False
 
@@ -166,7 +167,10 @@ class InterfaceBrowser:
 		   (_('Description'), SUMMARY),
 		   ('', None)]
 
-	def __init__(self, policy, widgets):
+	def __init__(self, driver, widgets):
+		self.driver = driver
+		self.config = driver.config
+
 		tree_view = widgets.get_widget('components')
 		tree_view.set_property('has-tooltip', True)
 		def callback(widget, x, y, keyboard_mode, tooltip):
@@ -183,14 +187,13 @@ class InterfaceBrowser:
 					col = self.columns[col_index][1]
 					row = self.model[path]
 					iface = row[InterfaceBrowser.INTERFACE]
-					main_feed = self.policy.config.iface_cache.get_feed(iface.uri)
+					main_feed = self.config.iface_cache.get_feed(iface.uri)
 					tooltip.set_text(get_tooltip_text(self, iface, main_feed, col))
 				return True
 			else:
 				return False
 		tree_view.connect('query-tooltip', callback)
 
-		self.policy = policy
 		self.cached_icon = {}	# URI -> GdkPixbuf
 		self.default_icon = tree_view.style.lookup_icon_set(gtk.STOCK_EXECUTE).render_icon(tree_view.style,
 			gtk.TEXT_DIR_NONE, gtk.STATE_NORMAL, gtk.ICON_SIZE_SMALL_TOOLBAR, tree_view, None)
@@ -246,11 +249,11 @@ class InterfaceBrowser:
 				return True
 			if bev.button != 1 or bev.type != gtk.gdk._2BUTTON_PRESS:
 				return False
-			properties.edit(policy, self.model[path][InterfaceBrowser.INTERFACE], self.compile, show_versions = True)
+			properties.edit(driver, self.model[path][InterfaceBrowser.INTERFACE], self.compile, show_versions = True)
 		tree_view.connect('button-press-event', button_press)
 
-		tree_view.connect('destroy', lambda s: policy.watchers.remove(self.build_tree))
-		policy.watchers.append(self.build_tree)
+		tree_view.connect('destroy', lambda s: driver.watchers.remove(self.build_tree))
+		driver.watchers.append(self.build_tree)
 
 	def set_root(self, root):
 		assert isinstance(root, model.Interface)
@@ -271,7 +274,7 @@ class InterfaceBrowser:
 			return self.cached_icon[iface.uri]
 		except KeyError:
 			# Try the on-disk cache
-			iconpath = self.policy.config.iface_cache.get_icon_path(iface)
+			iconpath = self.config.iface_cache.get_icon_path(iface)
 
 			if iconpath:
 				icon = load_icon(iconpath, ICON_SIZE, ICON_SIZE)
@@ -283,7 +286,10 @@ class InterfaceBrowser:
 			# Download a new icon if we don't have one, or if the
 			# user did a 'Refresh'
 			if iconpath is None or self.update_icons:
-				fetcher = self.policy.download_icon(iface)
+				if self.config.network_use == model.network_offline:
+					fetcher = None
+				else:
+					fetcher = self.config.fetcher.download_icon(iface)
 				if fetcher:
 					if iface.uri not in self.cached_icon:
 						self.cached_icon[iface.uri] = None	# Only try once
@@ -296,7 +302,7 @@ class InterfaceBrowser:
 							# Try to insert new icon into the cache
 							# If it fails, we'll be left with None in the cached_icon so
 							# we don't try again.
-							iconpath = self.policy.config.iface_cache.get_icon_path(iface)
+							iconpath = self.config.iface_cache.get_icon_path(iface)
 							if iconpath:
 								self.cached_icon[iface.uri] = load_icon(iconpath, ICON_SIZE, ICON_SIZE)
 								self.build_tree()
@@ -305,7 +311,7 @@ class InterfaceBrowser:
 						except Exception as ex:
 							import traceback
 							traceback.print_exc()
-							self.policy.handler.report_error(ex)
+							self.config.handler.report_error(ex)
 					update_display()
 				# elif fetcher is None: don't store anything in cached_icon
 
@@ -319,14 +325,14 @@ class InterfaceBrowser:
 		return None
 
 	def build_tree(self):
-		iface_cache = self.policy.config.iface_cache
+		iface_cache = self.config.iface_cache
 
 		if self.original_implementation is None:
 			self.set_original_implementations()
 
 		done = {}	# Detect cycles
 
-		sels = self.policy.solver.selections
+		sels = self.driver.solver.selections
 
 		self.model.clear()
 		def add_node(parent, iface, commands, essential):
@@ -358,7 +364,7 @@ class InterfaceBrowser:
 					version_str += _(' (was %s)') % old_impl.get_version()
 				self.model[iter][InterfaceBrowser.VERSION] = version_str
 
-				self.model[iter][InterfaceBrowser.DOWNLOAD_SIZE] = utils.get_fetch_info(self.policy, impl)
+				self.model[iter][InterfaceBrowser.DOWNLOAD_SIZE] = utils.get_fetch_info(self.config, impl)
 
 				deps = sel.dependencies
 				for c in commands:
@@ -387,12 +393,12 @@ class InterfaceBrowser:
 	def show_popup_menu(self, iface, bev):
 		import bugs
 
-		have_source =  properties.have_source_for(self.policy, iface)
+		have_source =  properties.have_source_for(self.config, iface)
 
 		menu = gtk.Menu()
-		for label, cb in [(_('Show Feeds'), lambda: properties.edit(self.policy, iface, self.compile)),
-				  (_('Show Versions'), lambda: properties.edit(self.policy, iface, self.compile, show_versions = True)),
-				  (_('Report a Bug...'), lambda: bugs.report_bug(self.policy, iface))]:
+		for label, cb in [(_('Show Feeds'), lambda: properties.edit(self.driver, iface, self.compile)),
+				  (_('Show Versions'), lambda: properties.edit(self.driver, iface, self.compile, show_versions = True)),
+				  (_('Report a Bug...'), lambda: bugs.report_bug(self.driver, iface))]:
 			item = gtk.MenuItem(label)
 			if cb:
 				item.connect('activate', lambda item, cb=cb: cb())
@@ -429,14 +435,14 @@ class InterfaceBrowser:
 			info(_("0compile command completed successfully. Reloading interface details."))
 			reader.update_from_cache(interface)
 			for feed in interface.extra_feeds:
-				 self.policy.config.iface_cache.get_feed(feed.uri, force = True)
+				self.config.iface_cache.get_feed(feed.uri, force = True)
 			import main
 			main.recalculate()
 		compile.compile(on_success, interface.uri, autocompile = autocompile)
 
 	def set_original_implementations(self):
 		assert self.original_implementation is None
-		self.original_implementation = self.policy.implementation.copy()
+		self.original_implementation = self.driver.solver.selections.copy()
 
 	def update_download_status(self, only_update_visible = False):
 		"""Called at regular intervals while there are downloads in progress,
@@ -446,13 +452,13 @@ class InterfaceBrowser:
 		# A download may be for a feed, an interface or an implementation.
 		# Create the reverse mapping (item -> download)
 		hints = {}
-		for dl in self.policy.handler.monitored_downloads:
+		for dl in self.config.handler.monitored_downloads:
 			if dl.hint:
 				if dl.hint not in hints:
 					hints[dl.hint] = []
 				hints[dl.hint].append(dl)
 
-		selections = self.policy.solver.selections
+		selections = self.driver.solver.selections
 
 		# Only update currently visible rows
 		if only_update_visible and self.tree_view.get_visible_range() != None:
@@ -464,14 +470,20 @@ class InterfaceBrowser:
 			firstVisibleIter = self.model.get_iter_root()
 			lastVisiblePath = None
 
+		solver = self.driver.solver
+		requirements = self.driver.requirements
+		iface_cache = self.config.iface_cache
+
 		for it in walk(self.model, firstVisibleIter):
 			row = self.model[it]
 			iface = row[InterfaceBrowser.INTERFACE]
 
 			# Is this interface the download's hint?
 			downloads = hints.get(iface, [])	# The interface itself
-		     	downloads += hints.get(iface.uri, [])	# The main feed
-			for feed in self.policy.usable_feeds(iface):
+			downloads += hints.get(iface.uri, [])	# The main feed
+
+			arch = solver.get_arch_for(requirements, iface)
+			for feed in iface_cache.usable_feeds(iface, arch):
 				downloads += hints.get(feed.uri, []) # Other feeds
 			impl = selections.get(iface, None)
 			if impl:
@@ -496,7 +508,7 @@ class InterfaceBrowser:
 					values_dict = {'downloaded': pretty_size(so_far), 'number': len(downloads)}
 				row[InterfaceBrowser.SUMMARY] = summary % values_dict
 			else:
-				row[InterfaceBrowser.DOWNLOAD_SIZE] = utils.get_fetch_info(self.policy, impl)
+				row[InterfaceBrowser.DOWNLOAD_SIZE] = utils.get_fetch_info(self.config, impl)
 				row[InterfaceBrowser.SUMMARY] = iface.summary
 
 			if self.model.get_path(it) == lastVisiblePath:
@@ -507,7 +519,7 @@ class InterfaceBrowser:
 		for it in walk(self.model, self.model.get_iter_root()):
 			row = self.model[it]
 			iface = row[InterfaceBrowser.INTERFACE]
-			sel = self.policy.solver.selections.selections.get(iface.uri, None)
+			sel = self.driver.solver.selections.selections.get(iface.uri, None)
 
 			if sel is None and row[InterfaceBrowser.PROBLEM]:
 				row[InterfaceBrowser.BACKGROUND] = '#f88'
