@@ -5,9 +5,10 @@ import gtk, gobject, os, pango
 from zeroinstall import _
 from zeroinstall.injector import model, writer
 from zeroinstall import support
+from zeroinstall.gtkui import gtkutils
 import utils
 
-def _build_stability_menu(policy, impl):
+def _build_stability_menu(impl):
 	menu = gtk.Menu()
 
 	upstream = impl.upstream_stability or model.testing
@@ -55,15 +56,15 @@ NOTES = 7
 WEIGHT = 8	# Selected item is bold
 LANGS = 9
 
-def get_tooltip_text(policy, interface, impl):
+def get_tooltip_text(config, interface, impl):
 	if impl.local_path:
 		return _("Local: %s") % impl.local_path
 	if impl.id.startswith('package:'):
 		return _("Native package: %s") % impl.id.split(':', 1)[1]
-	if policy.get_cached(impl):
-		return _("Cached: %s") % policy.get_implementation_path(impl)
+	if impl.is_available(config.stores):
+		return _("Cached: %s") % config.stores.lookup_any(impl.digests)
 
-	src = policy.fetcher.get_best_source(impl)
+	src = config.fetcher.get_best_source(impl)
 	if src:
 		size = support.pretty_size(src.size)
 		return _("Not yet downloaded (%s)") % size
@@ -74,11 +75,11 @@ class ImplementationList:
 	tree_view = None
 	model = None
 	interface = None
-	policy = None
+	driver = None
 
-	def __init__(self, policy, interface, widgets):
+	def __init__(self, driver, interface, widgets):
 		self.interface = interface
-		self.policy = policy
+		self.driver = driver
 
 		self.model = gtk.ListStore(object, str, str, str,	# Item, arch, stability, version,
 			   str, gobject.TYPE_BOOLEAN, str, str,		# fetch, unusable, released, notes,
@@ -110,7 +111,7 @@ class ImplementationList:
 				path = pos[0]
 				row = self.model[path]
 				if row[ITEM]:
-					tooltip.set_text(get_tooltip_text(policy, interface, row[ITEM]))
+					tooltip.set_text(get_tooltip_text(driver.config, interface, row[ITEM]))
 					return True
 			return False
 		self.tree_view.connect('query-tooltip', tooltip_callback)
@@ -127,36 +128,45 @@ class ImplementationList:
 			menu = gtk.Menu()
 
 			stability_menu = gtk.MenuItem(_('Rating'))
-			stability_menu.set_submenu(_build_stability_menu(self.policy, impl))
+			stability_menu.set_submenu(_build_stability_menu(impl))
 			stability_menu.show()
 			menu.append(stability_menu)
 
-			if not impl.id.startswith('package:') and self.policy.get_cached(impl):
+			if not impl.id.startswith('package:') and impl.is_available(self.driver.config.stores):
 				def open():
 					os.spawnlp(os.P_WAIT, '0launch',
 						'0launch', rox_filer, '-d',
-						self.policy.get_implementation_path(impl))
+						impl.local_path or self.driver.config.stores.lookup_any(impl.digests))
 				item = gtk.MenuItem(_('Open cached copy'))
 				item.connect('activate', lambda item: open())
 				item.show()
 				menu.append(item)
 
+			item = gtk.MenuItem(_('Explain this decision'))
+			item.connect('activate', lambda item: self.show_explaination(impl))
+			item.show()
+			menu.append(item)
+
 			menu.popup(None, None, None, bev.button, bev.time)
 
 		self.tree_view.connect('button-press-event', button_press)
 	
+	def show_explaination(self, impl):
+		reason = self.driver.solver.justify_decision(self.driver.requirements, self.interface, impl)
+		gtkutils.show_message_box(self.tree_view.get_toplevel(), reason, gtk.MESSAGE_INFO)
+	
 	def get_selection(self):
 		return self.tree_view.get_selection()
 	
 	def set_items(self, items):
 		self.model.clear()
-		selected = self.policy.solver.selections.get(self.interface, None)
+		selected = self.driver.solver.selections.get(self.interface, None)
 		for item, unusable in items:
 			new = self.model.append()
 			self.model[new][ITEM] = item
 			self.model[new][VERSION] = item.get_version()
 			self.model[new][RELEASED] = item.released or "-"
-			self.model[new][FETCH] = utils.get_fetch_info(self.policy, item)
+			self.model[new][FETCH] = utils.get_fetch_info(self.driver.config, item)
 			if item.user_stability:
 				if item.user_stability == model.insecure:
 					self.model[new][STABILITY] = _('INSECURE')
