@@ -29,8 +29,8 @@ def format_para(para):
 	lines = [l.strip() for l in para.split('\n')]
 	return ' '.join(lines)
 
-def have_source_for(policy, interface):
-	iface_cache = policy.config.iface_cache
+def have_source_for(config, interface):
+	iface_cache = config.iface_cache
 	# Note: we don't want to actually fetch the source interfaces at
 	# this point, so we check whether:
 	# - We have a feed of type 'src' (not fetched), or
@@ -128,37 +128,39 @@ class Description:
 			buffer.insert(iter, _('Homepage: '))
 			buffer.insert_with_tags(iter, '%s\n' % x.content, self.link_style)
 
-		buffer.insert_with_tags(iter, '\n' + _('Signatures') + '\n', heading_style)
-		sigs = iface_cache.get_cached_signatures(feed.url)
-		if sigs:
-			for sig in sigs:
-				if isinstance(sig, gpg.ValidSig):
-					name = _('<unknown>')
-					details = sig.get_details()
-					for item in details:
-						if item[0] == 'uid' and len(item) > 9:
-							name = item[9]
-							break
-					buffer.insert_with_tags(iter, _('Valid signature by "%(name)s"\n- Dated: %(sig_date)s\n- Fingerprint: %(sig_fingerprint)s\n') %
-							{'name': name, 'sig_date': time.strftime('%c', time.localtime(sig.get_timestamp())), 'sig_fingerprint': sig.fingerprint})
-					if not sig.is_trusted():
-						if os.path.isabs(feed.url):
-							buffer.insert_with_tags(iter, _('WARNING: This key is not in the trusted list') + '\n')
-						else:
-							buffer.insert_with_tags(iter, _('WARNING: This key is not in the trusted list (either you removed it, or '
-											'you trust one of the other signatures)') + '\n')
-				else:
-					buffer.insert_with_tags(iter, '%s\n' % sig)
-		else:
-			buffer.insert_with_tags(iter, _('No signature information (old style feed or out-of-date cache)') + '\n')
+		if feed.local_path is None:
+			buffer.insert_with_tags(iter, '\n' + _('Signatures') + '\n', heading_style)
+			sigs = iface_cache.get_cached_signatures(feed.url)
+			if sigs:
+				for sig in sigs:
+					if isinstance(sig, gpg.ValidSig):
+						name = _('<unknown>')
+						details = sig.get_details()
+						for item in details:
+							if item[0] == 'uid' and len(item) > 9:
+								name = item[9]
+								break
+						buffer.insert_with_tags(iter, _('Valid signature by "%(name)s"\n- Dated: %(sig_date)s\n- Fingerprint: %(sig_fingerprint)s\n') %
+								{'name': name, 'sig_date': time.strftime('%c', time.localtime(sig.get_timestamp())), 'sig_fingerprint': sig.fingerprint})
+						if not sig.is_trusted():
+							if os.path.isabs(feed.url):
+								buffer.insert_with_tags(iter, _('WARNING: This key is not in the trusted list') + '\n')
+							else:
+								buffer.insert_with_tags(iter, _('WARNING: This key is not in the trusted list (either you removed it, or '
+												'you trust one of the other signatures)') + '\n')
+					else:
+						buffer.insert_with_tags(iter, '%s\n' % sig)
+			else:
+				buffer.insert_with_tags(iter, _('No signature information (old style feed or out-of-date cache)') + '\n')
 
 class Feeds:
 	URI = 0
 	ARCH = 1
 	USED = 2
 
-	def __init__(self, policy, interface, widgets):
-		self.policy = policy
+	def __init__(self, config, arch, interface, widgets):
+		self.config = config
+		self.arch = arch
 		self.interface = interface
 
 		self.model = gtk.ListStore(str, str, bool)
@@ -170,10 +172,10 @@ class Feeds:
 			self.model.append(line)
 
 		add_remote_feed_button = widgets.get_widget('add_remote_feed')
-		add_remote_feed_button.connect('clicked', lambda b: add_remote_feed(policy, widgets.get_widget(), interface))
+		add_remote_feed_button.connect('clicked', lambda b: add_remote_feed(config, widgets.get_widget(), interface))
 
 		add_local_feed_button = widgets.get_widget('add_local_feed')
-		add_local_feed_button.connect('clicked', lambda b: add_local_feed(policy, interface))
+		add_local_feed_button.connect('clicked', lambda b: add_local_feed(config, interface))
 
 		self.remove_feed_button = widgets.get_widget('remove_feed')
 		def remove_feed(button):
@@ -188,7 +190,7 @@ class Feeds:
 						main.recalculate()
 						return
 					else:
-						dialog.alert(self.get_toplevel(),
+						dialog.alert(self.remove_feed_button.get_toplevel(),
 							_("Can't remove '%s' as you didn't add it.") % feed_uri)
 						return
 			raise Exception(_("Missing feed '%s'!") % feed_uri)
@@ -206,9 +208,9 @@ class Feeds:
 		sel.select_path((0,))
 	
 	def build_model(self):
-		iface_cache = self.policy.config.iface_cache
+		iface_cache = self.config.iface_cache
 
-		usable_feeds = frozenset(self.policy.usable_feeds(self.interface))
+		usable_feeds = frozenset(self.config.iface_cache.usable_feeds(self.interface, self.arch))
 		unusable_feeds = frozenset(iface_cache.get_feed_imports(self.interface)) - usable_feeds
 
 		out = [[self.interface.uri, None, True]]
@@ -220,7 +222,7 @@ class Feeds:
 		return out
 
 	def sel_changed(self, sel):
-		iface_cache = self.policy.config.iface_cache
+		iface_cache = self.config.iface_cache
 
 		model, miter = sel.get_selected()
 		if not miter: return	# build in progress
@@ -229,9 +231,10 @@ class Feeds:
 		enable_remove = False
 		for x in self.interface.extra_feeds:
 			if x.uri == feed_url:
-				if x.user_override:
+				if x.user_override and not x.site_package:
 					enable_remove = True
-		self.remove_feed_button.set_sensitive( enable_remove )
+					break
+		self.remove_feed_button.set_sensitive(enable_remove)
 		try:
 			self.description.set_details(iface_cache, iface_cache.get_feed(feed_url))
 		except zeroinstall.SafeException as ex:
@@ -252,10 +255,10 @@ class Properties:
 	interface = None
 	use_list = None
 	window = None
-	policy = None
+	driver = None
 
-	def __init__(self, policy, interface, compile, show_versions = False):
-		self.policy = policy
+	def __init__(self, driver, interface, compile, show_versions = False):
+		self.driver = driver
 
 		widgets = Template('interface_properties')
 
@@ -280,7 +283,8 @@ class Properties:
 		notebook = widgets.get_widget('interface_notebook')
 		assert notebook
 
-		feeds = Feeds(policy, interface, widgets)
+		target_arch = self.driver.solver.get_arch_for(driver.requirements, interface = interface)
+		feeds = Feeds(driver.config, target_arch, interface, widgets)
 
 		stability = widgets.get_widget('preferred_stability')
 		stability.set_active(0)
@@ -306,7 +310,7 @@ class Properties:
 			main.recalculate()
 		stability.connect('changed', set_stability_policy)
 
-		self.use_list = ImplementationList(policy, interface, widgets)
+		self.use_list = ImplementationList(driver, interface, widgets)
 
 		self.update_list()
 
@@ -316,8 +320,8 @@ class Properties:
 			self.update_list()
 			feeds.updated()
 			self.shade_compile()
-		window.connect('destroy', lambda s: policy.watchers.remove(updated))
-		policy.watchers.append(updated)
+		window.connect('destroy', lambda s: driver.watchers.remove(updated))
+		driver.watchers.append(updated)
 		self.shade_compile()
 
 		if show_versions:
@@ -330,10 +334,10 @@ class Properties:
 		self.window.destroy()
 	
 	def shade_compile(self):
-		self.compile_button.set_sensitive(have_source_for(self.policy, self.interface))
+		self.compile_button.set_sensitive(have_source_for(self.driver.config, self.interface))
 	
 	def update_list(self):
-		ranked_items = self.policy.solver.details.get(self.interface, None)
+		ranked_items = self.driver.solver.details.get(self.interface, None)
 		if ranked_items is None:
 			# The Solver didn't get this far, but we should still display them!
 			ranked_items = [(impl, _("(solve aborted before here)"))
@@ -343,9 +347,9 @@ class Properties:
 		self.use_list.set_items(ranked_items)
 
 @tasks.async
-def add_remote_feed(policy, parent, interface):
+def add_remote_feed(config, parent, interface):
 	try:
-		iface_cache = policy.config.iface_cache
+		iface_cache = config.iface_cache
 
 		d = gtk.MessageDialog(parent, 0, gtk.MESSAGE_QUESTION, gtk.BUTTONS_CANCEL,
 			_('Enter the URL of the new source of implementations of this interface:'))
@@ -388,7 +392,7 @@ def add_remote_feed(policy, parent, interface):
 					url = entry.get_text()
 					if not url:
 						raise zeroinstall.SafeException(_('Enter a URL'))
-					fetch = policy.fetcher.download_and_import_feed(url, iface_cache)
+					fetch = config.fetcher.download_and_import_feed(url, iface_cache)
 					if fetch:
 						d.set_sensitive(False)
 						yield fetch
@@ -422,14 +426,14 @@ def add_remote_feed(policy, parent, interface):
 	except Exception as ex:
 		import traceback
 		traceback.print_exc()
-		policy.handler.report_error(ex)
+		config.handler.report_error(ex)
 
-def add_local_feed(policy, interface):
+def add_local_feed(config, interface):
 	chooser = gtk.FileChooserDialog(_('Select XML feed file'), action=gtk.FILE_CHOOSER_ACTION_OPEN, buttons=(gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL, gtk.STOCK_OPEN, gtk.RESPONSE_OK))
 	def ok(feed):
 		from zeroinstall.injector import reader
 		try:
-			feed_targets = policy.get_feed_targets(feed)
+			feed_targets = config.iface_cache.get_feed_targets(feed)
 			if interface not in feed_targets:
 				raise Exception(_("Not a valid feed for '%(uri)s'; this is a feed for:\n%(feed_for)s") %
 						{'uri': interface.uri,
@@ -456,11 +460,11 @@ def add_local_feed(policy, interface):
 	chooser.connect('response', check_response)
 	chooser.show()
 
-def edit(policy, interface, compile, show_versions = False):
+def edit(driver, interface, compile, show_versions = False):
 	assert isinstance(interface, Interface)
 	if interface in _dialogs:
 		_dialogs[interface].destroy()
-	_dialogs[interface] = Properties(policy, interface, compile, show_versions = show_versions)
+	_dialogs[interface] = Properties(driver, interface, compile, show_versions = show_versions)
 	_dialogs[interface].show()
 
 properties_help = help_box.HelpBox(_("Injector Properties Help"),
@@ -510,7 +514,11 @@ is provided by your distribution's package manager, not by Zero Install. \
 In off-line mode, only cached implementations are considered for use.
 
 Arch indicates what kind of computer system the implementation is for, or 'any' \
-if it works with all types of system.""") + '\n'),
+if it works with all types of system.
+
+If you want to know why a particular version wasn't chosen, right-click over it \
+and choose "Explain this decision" from the popup menu.
+""") + '\n'),
 (_('Sort order'), '\n' +
 _("""The implementations are ordered by version number (highest first), with the \
 currently selected one in bold. This is the "best" usable version.
