@@ -8,6 +8,7 @@ The B{0install run} command-line interface.
 import sys
 
 from zeroinstall import _
+from zeroinstall.support import tasks
 from zeroinstall.cmd import UsageError, select
 from zeroinstall.injector import model
 
@@ -22,7 +23,7 @@ def add_options(parser):
 def handle(config, options, args):
 	if len(args) < 1:
 		raise UsageError()
-	iface_uri = model.canonical_iface_uri(args[0])
+
 	prog_args = args[1:]
 
 	def test_callback(sels):
@@ -31,11 +32,21 @@ def handle(config, options, args):
 					     False,	# dry-run
 					     options.main)
 
-	sels = select.get_selections(config, options, iface_uri,
-				select_only = False, download_only = False,
-				test_callback = test_callback)
-	if not sels:
-		sys.exit(1)	# Aborted by user
+	app = config.app_mgr.lookup_app(args[0], missing_ok = True)
+	if app is not None:
+		sels = app.get_selections()
+		dl = app.download_selections(sels)
+		if dl:
+			tasks.wait_for_blocker(dl)
+			tasks.check(dl)
+	else:
+		iface_uri = model.canonical_iface_uri(args[0])
+
+		sels = select.get_selections(config, options, iface_uri,
+					select_only = False, download_only = False,
+					test_callback = test_callback)
+		if not sels:
+			sys.exit(1)	# Aborted by user
 
 	from zeroinstall.injector import run
 	run.execute_selections(sels, prog_args, dry_run = options.dry_run, main = options.main, wrapper = options.wrapper, stores = config.stores)
