@@ -1,6 +1,6 @@
 """
 Check for updates in a background process. If we can start a program immediately, but some of our information
-is rather old (longer that the L{policy.Policy.freshness} threshold) then we run it anyway, and check for updates using a new
+is rather old (longer that the L{config.Config.freshness} threshold) then we run it anyway, and check for updates using a new
 process that runs quietly in the background.
 
 This avoids the need to annoy people with a 'checking for updates' box when they're trying to run things.
@@ -18,11 +18,6 @@ from zeroinstall.injector import handler
 def _escape_xml(s):
 	return s.replace('&', '&amp;').replace('<', '&lt;')
 
-def _exec_gui(uri, *args):
-	parent_dir = os.path.dirname(os.path.dirname(__file__))
-	child_args = [sys.executable, '-u', os.path.join(parent_dir, '0launch-gui/0launch-gui')] + list(args) + [uri]
-	os.execvp(sys.executable, child_args)
-
 class _NetworkState:
 	NM_STATE_UNKNOWN = 0
 	NM_STATE_ASLEEP = 10
@@ -42,7 +37,6 @@ class _NetworkState:
 		4: NM_STATE_DISCONNECTED,
 	}
 
-
 class BackgroundHandler(handler.Handler):
 	"""A Handler for non-interactive background updates. Runs the GUI if interaction is required."""
 	def __init__(self, title, root):
@@ -52,6 +46,7 @@ class BackgroundHandler(handler.Handler):
 		self.network_manager = None
 		self.notification_service_caps = []
 		self.root = root	# If we need to confirm any keys, run the GUI on this
+		self.need_gui = False
 
 		try:
 			import dbus
@@ -106,8 +101,20 @@ class BackgroundHandler(handler.Handler):
 
 	def confirm_import_feed(self, pending, valid_sigs):
 		"""Run the GUI if we need to confirm any keys."""
-		info(_("Can't update feed; signature not yet trusted. Running GUI..."))
-		_exec_gui(self.root, '--download', '--refresh', '--systray')
+
+		if os.environ.get('DISPLAY', None):
+			info(_("Can't update feed; signature not yet trusted. Running GUI..."))
+
+			self.need_gui = True
+
+			for dl in self.monitored_downloads:
+				dl.abort()
+
+			raise handler.NoTrustedKeys("need to switch to GUI to confirm keys")
+		else:
+			raise handler.NoTrustedKeys(_("Background update for {iface} needed to confirm keys, but no GUI available!").format(
+					iface = self.root))
+
 
 	def report_error(self, exception, tb = None):
 		from zeroinstall.injector import download
@@ -166,7 +173,7 @@ def _detach():
 	# Note: this only fixes the most common case; it could be waiting
 	# on any other FD as well. We should really use gobject.spawn_async
 	# to close *all* FDs.
-	null = os.open('/dev/null', os.O_RDWR)
+	null = os.open(os.devnull, os.O_RDWR)
 	os.dup2(null, 1)
 	os.close(null)
 
@@ -176,8 +183,11 @@ def _detach():
 
 	return False
 
-def _check_for_updates(requirements, verbose):
-	from zeroinstall.injector.policy import Policy
+def _check_for_updates(requirements, verbose, app):
+	if app is not None:
+		old_sels = app.get_selections()
+
+	from zeroinstall.injector.driver import Driver
 	from zeroinstall.injector.config import load_config
 
 	background_handler = BackgroundHandler(requirements.interface_uri, requirements.interface_uri)
@@ -185,13 +195,13 @@ def _check_for_updates(requirements, verbose):
 	root_iface = background_config.iface_cache.get_interface(requirements.interface_uri).get_name()
 	background_handler.title = root_iface
 
-	policy = Policy(config = background_config, requirements = requirements)
+	driver = Driver(config = background_config, requirements = requirements)
 
 	info(_("Checking for updates to '%s' in a background process"), root_iface)
 	if verbose:
-		policy.handler.notify("Zero Install", _("Checking for updates to '%s'...") % root_iface, timeout = 1)
+		background_handler.notify("Zero Install", _("Checking for updates to '%s'...") % root_iface, timeout = 1)
 
-	network_state = policy.handler.get_network_state()
+	network_state = background_handler.get_network_state()
 	if network_state not in (_NetworkState.NM_STATE_CONNECTED_SITE, _NetworkState.NM_STATE_CONNECTED_GLOBAL):
 		info(_("Not yet connected to network (status = %d). Sleeping for a bit..."), network_state)
 		import time
@@ -202,28 +212,43 @@ def _check_for_updates(requirements, verbose):
 	else:
 		info(_("NetworkManager says we're on-line. Good!"))
 
-	policy.freshness = 0			# Don't bother trying to refresh when getting the interface
-	refresh = policy.refresh_all()		# (causes confusing log messages)
+	background_config.freshness = 0			# Don't bother trying to refresh when getting the interface
+	refresh = driver.solve_with_downloads(force = True)	# (causes confusing log messages)
 	tasks.wait_for_blocker(refresh)
 
-	# We could even download the archives here, but for now just
-	# update the interfaces.
+	if background_handler.need_gui or driver.get_uncached_implementations():
+		background_handler.notify("Zero Install",
+				      _("Updates ready to download for '%s'.") % root_iface,
+				      timeout = 1)
 
-	if not policy.need_download():
+		if os.environ.get('DISPLAY', None):
+			# Run the GUI...
+			from zeroinstall import helpers
+			gui_args = ['--refresh', '--systray', '--download'] + requirements.get_as_options()
+			new_sels = helpers.get_selections_gui(requirements.interface_uri, gui_args)
+			if new_sels is None:
+				sys.exit(0)	# Cancelled by user
+		else:
+			tasks.wait_for_blocker(driver.download_uncached_implementations())
+			new_sels = driver.solver.selections
+	else:
 		if verbose:
-			policy.handler.notify("Zero Install", _("No updates to download."), timeout = 1)
-		sys.exit(0)
+			background_handler.notify("Zero Install", _("No updates to download."), timeout = 1)
+		new_sels = driver.solver.selections
+
+	if app is not None:
+		assert driver.solver.ready
+		from zeroinstall.support import xmltools
+		if not xmltools.nodes_equal(new_sels.toDOM(), old_sels.toDOM()):
+			app.set_selections(new_sels)
+		app.set_last_checked()
+	sys.exit(0)
 
-	policy.handler.notify("Zero Install",
-			      _("Updates ready to download for '%s'.") % root_iface,
-			      timeout = 1)
-	_exec_gui(policy.root, '--refresh', '--systray')
-	sys.exit(1)
 
 def spawn_background_update(driver, verbose):
 	"""Spawn a detached child process to check for updates.
 	@param driver: driver containing interfaces to update
-	@type policy: L{driver.Driver}
+	@type driver: L{driver.Driver}
 	@param verbose: whether to notify the user about minor events
 	@type verbose: bool
 	@since: 1.5 (used to take a Policy)"""
@@ -234,12 +259,23 @@ def spawn_background_update(driver, verbose):
 	for uri in driver.solver.feeds_used:
 		iface_cache.mark_as_checking(uri)
 
+	spawn_background_update2(driver.requirements, verbose)
+
+def spawn_background_update2(requirements, verbose, app = None):
+	"""Spawn a detached child process to check for updates.
+	@param requirements: requirements for the new selections
+	@type requirements: L{requirements.Requirements}
+	@param verbose: whether to notify the user about minor events
+	@type verbose: bool
+	@param app: application to update (if any)
+	@type app: L{apps.App} | None
+	@since: 1.9"""
 	if _detach():
 		return
 
 	try:
 		try:
-			_check_for_updates(driver.requirements, verbose)
+			_check_for_updates(requirements, verbose, app)
 		except SystemExit:
 			raise
 		except:
