@@ -10,7 +10,7 @@ from zeroinstall import _
 import os, platform, re, subprocess, sys
 from logging import warn, info
 from zeroinstall.injector import namespaces, model, arch
-from zeroinstall.support import basedir
+from zeroinstall.support import basedir, portable_rename
 
 _dotted_ints = '[0-9]+(?:\.[0-9]+)*'
 
@@ -18,8 +18,8 @@ _dotted_ints = '[0-9]+(?:\.[0-9]+)*'
 _zeroinstall_regexp = '(?:%s)(?:-(?:pre|rc|post|)(?:%s))*' % (_dotted_ints, _dotted_ints)
 
 # This matches the interesting bits of distribution version numbers
-# (first bit is for Java-style 6b17 syntax)
-_version_regexp = '({ints}b)?({zero})(-r{ints})?'.format(zero = _zeroinstall_regexp, ints = _dotted_ints)
+# (first matching group is for Java-style 6b17 syntax, or "major")
+_version_regexp = '(?:[a-z])?({ints}b)?({zero})(-r{ints})?'.format(zero = _zeroinstall_regexp, ints = _dotted_ints)
 
 # We try to do updates atomically without locking, but we don't worry too much about
 # duplicate entries or being a little out of sync with the on-disk copy.
@@ -56,7 +56,8 @@ class Cache(object):
 		while data:
 			wrote = os.write(tmp, data)
 			data = data[wrote:]
-		os.rename(tmp_name, os.path.join(self.cache_dir, self.cache_leaf))
+		os.close(tmp)
+		portable_rename(tmp_name, os.path.join(self.cache_dir, self.cache_leaf))
 
 		self._load_cache()
 
@@ -64,8 +65,7 @@ class Cache(object):
 	# Throws an exception if the cache doesn't exist or has the wrong format.
 	def _load_cache(self):
 		self.cache = cache = {}
-		stream = open(os.path.join(self.cache_dir, self.cache_leaf))
-		try:
+		with open(os.path.join(self.cache_dir, self.cache_leaf)) as stream:
 			for line in stream:
 				line = line.strip()
 				if not line:
@@ -79,8 +79,6 @@ class Cache(object):
 			for line in stream:
 				key, value = line.split('=', 1)
 				cache[key] = value[:-1]
-		finally:
-			stream.close()
 
 	# Check the source file hasn't changed since we created the cache
 	def _check_valid(self):
@@ -106,11 +104,8 @@ class Cache(object):
 		cache_path = os.path.join(self.cache_dir, self.cache_leaf)
 		self.cache[key] = value
 		try:
-			stream = open(cache_path, 'a')
-			try:
+			with open(cache_path, 'a') as stream:
 				stream.write('%s=%s\n' % (key, value))
-			finally:
-				stream.close()
 		except Exception as ex:
 			warn("Failed to write to cache %s: %s=%s: %s", cache_path, key, value, ex)
 
@@ -188,17 +183,20 @@ class Distribution(object):
 					if only_if_missing:
 						return None
 					warn(_("Duplicate ID '%s' for DistributionImplementation"), id)
-				impl = model.DistributionImplementation(feed, id, self)
+				impl = model.DistributionImplementation(feed, id, self, item)
 				feed.implementations[id] = impl
 
 				impl.installed = installed
 				impl.metadata = item_attrs
 
-				item_main = item_attrs.get('main', None)
-				if item_main and not item_main.startswith('/'):
-					raise model.InvalidInterface(_("'main' attribute must be absolute, but '%s' doesn't start with '/'!") %
-								item_main)
-				impl.main = item_main
+				if 'run' not in impl.commands:
+					item_main = item_attrs.get('main', None)
+					if item_main:
+						if item_main.startswith('/'):
+							impl.main = item_main
+						else:
+							raise model.InvalidInterface(_("'main' attribute must be absolute, but '%s' doesn't start with '/'!") %
+										item_main)
 				impl.upstream_stability = model.packaged
 
 				return impl
@@ -242,10 +240,7 @@ class Distribution(object):
 class WindowsDistribution(Distribution):
 	def get_package_info(self, package, factory):
 		def _is_64bit_windows():
-			import sys
 			p = sys.platform
-			import platform
-			bits, linkage = platform.architecture()
 			from win32process import IsWow64Process
 			if p == 'win64' or (p == 'win32' and IsWow64Process()): return True
 			elif p == 'win32': return False
@@ -279,41 +274,30 @@ class WindowsDistribution(Distribution):
 				value64 = ''
 			return (value32, value64)
 
+		def find_java(part, win_version, zero_version):
+			reg_path = r"SOFTWARE\JavaSoft\{part}\{win_version}".format(part = part, win_version = win_version)
+			(java32_home, java64_home) = _read_hklm_reg(reg_path, "JavaHome")
+
+			for (home, arch) in [(java32_home, 'i486'),
+					     (java64_home, 'x86_64')]:
+				if os.path.isfile(home + r"\bin\java.exe"):
+					impl = factory('package:windows:%s:%s:%s' % (package, zero_version, arch))
+					impl.machine = arch
+					impl.version = model.parse_version(zero_version)
+					impl.upstream_stability = model.packaged
+					impl.main = home + r"\bin\java.exe"
+
 		if package == 'openjdk-6-jre':
-			(java32_home, java64_home) = _read_hklm_reg(r"SOFTWARE\JavaSoft\Java Runtime Environment\1.6", "JavaHome")
-			import os.path
+			find_java("Java Runtime Environment", "1.6", '6')
+		elif package == 'openjdk-6-jdk':
+			find_java("Java Development Kit", "1.6", '6')
+		elif package == 'openjdk-7-jre':
+			find_java("Java Runtime Environment", "1.7", '7')
+		elif package == 'openjdk-7-jdk':
+			find_java("Java Development Kit", "1.7", '7')
 
-			if os.path.isfile(java32_home + r"\bin\java.exe"):
-				impl = factory('package:windows:%s:%s:%s' % (package, '6', 'i486'))
-				impl.machine = 'i486';
-				impl.version = model.parse_version('6')
-				impl.upstream_stability = model.packaged
-				impl.main = java32_home + r"\bin\java.exe"
-
-			if os.path.isfile(java64_home + r"\bin\java.exe"):
-				impl = factory('package:windows:%s:%s:%s' % (package, '6', 'x86_64'))
-				impl.machine = 'x86_64';
-				impl.version = model.parse_version('6')
-				impl.upstream_stability = model.packaged
-				impl.main = java64_home + r"\bin\java.exe"
-
-		if package == 'openjdk-6-jdk':
-			(java32_home, java64_home) = _read_hklm_reg(r"SOFTWARE\JavaSoft\Java Development Kit\1.6", "JavaHome")
-			import os.path
-
-			if os.path.isfile(java32_home + r"\bin\java.exe"):
-				impl = factory('package:windows:%s:%s:%s' % (package, '6', 'i486'))
-				impl.machine = 'i486';
-				impl.version = model.parse_version('6')
-				impl.upstream_stability = model.packaged
-				impl.main = java32_home + r"\bin\java.exe"
-
-			if os.path.isfile(java64_home + r"\bin\java.exe"):
-				impl = factory('package:windows:%s:%s:%s' % (package, '6', 'x86_64'))
-				impl.machine = 'x86_64';
-				impl.version = model.parse_version('6')
-				impl.upstream_stability = model.packaged
-				impl.main = java64_home + r"\bin\java.exe"
+	def get_score(self, disto_name):
+		return int(disto_name == 'Windows')
 
 class CachedDistribution(Distribution):
 	"""For distributions where querying the package database is slow (e.g. requires running
@@ -343,33 +327,32 @@ class CachedDistribution(Distribution):
 	def _load_cache(self):
 		"""Load {cache_leaf} cache file into self.versions if it is available and up-to-date.
 		Throws an exception if the cache should be (re)created."""
-		stream = open(os.path.join(self.cache_dir, self.cache_leaf))
-
-		cache_version = None
-		for line in stream:
-			if line == '\n':
-				break
-			name, value = line.split(': ')
-			if name == 'mtime' and int(value) != int(self._status_details.st_mtime):
-				raise Exception(_("Modification time of package database file has changed"))
-			if name == 'size' and int(value) != self._status_details.st_size:
-				raise Exception(_("Size of package database file has changed"))
-			if name == 'version':
-				cache_version = int(value)
-		else:
-			raise Exception(_('Invalid cache format (bad header)'))
-
-		if cache_version is None:
-			raise Exception(_('Old cache format'))
-
-		versions = self.versions
-		for line in stream:
-			package, version, zi_arch = line[:-1].split('\t')
-			versionarch = (version, intern(zi_arch))
-			if package not in versions:
-				versions[package] = [versionarch]
+		with open(os.path.join(self.cache_dir, self.cache_leaf)) as stream:
+			cache_version = None
+			for line in stream:
+				if line == '\n':
+					break
+				name, value = line.split(': ')
+				if name == 'mtime' and int(value) != int(self._status_details.st_mtime):
+					raise Exception(_("Modification time of package database file has changed"))
+				if name == 'size' and int(value) != self._status_details.st_size:
+					raise Exception(_("Size of package database file has changed"))
+				if name == 'version':
+					cache_version = int(value)
 			else:
-				versions[package].append(versionarch)
+				raise Exception(_('Invalid cache format (bad header)'))
+
+			if cache_version is None:
+				raise Exception(_('Old cache format'))
+
+			versions = self.versions
+			for line in stream:
+				package, version, zi_arch = line[:-1].split('\t')
+				versionarch = (version, intern(zi_arch))
+				if package not in versions:
+					versions[package] = [versionarch]
+				else:
+					versions[package].append(versionarch)
 
 	def _write_cache(self, cache):
 		#cache.sort() 	# Might be useful later; currently we don't care
@@ -386,7 +369,7 @@ class CachedDistribution(Distribution):
 				stream.write(line + '\n')
 			stream.close()
 
-			os.rename(tmpname,
+			portable_rename(tmpname,
 				  os.path.join(self.cache_dir,
 					       self.cache_leaf))
 		except:
@@ -422,12 +405,12 @@ class DebianDistribution(Distribution):
 
 	cache_leaf = 'dpkg-status.cache'
 
-	def __init__(self, dpkg_status, pkgcache):
+	def __init__(self, dpkg_status):
 		self.dpkg_cache = Cache('dpkg-status.cache', dpkg_status, 2)
 		self.apt_cache = {}
 
 	def _query_installed_package(self, package):
-		null = os.open('/dev/null', os.O_WRONLY)
+		null = os.open(os.devnull, os.O_WRONLY)
 		child = subprocess.Popen(["dpkg-query", "-W", "--showformat=${Version}\t${Architecture}\t${Status}\n", "--", package],
 						stdout = subprocess.PIPE, stderr = null)
 		os.close(null)
@@ -502,7 +485,7 @@ class DebianDistribution(Distribution):
 		for package in package_names:
 			# Check to see whether we could get a newer version using apt-get
 			try:
-				null = os.open('/dev/null', os.O_WRONLY)
+				null = os.open(os.devnull, os.O_WRONLY)
 				child = subprocess.Popen(['apt-cache', 'show', '--no-all-versions', '--', package], stdout = subprocess.PIPE, stderr = null)
 				os.close(null)
 
@@ -729,7 +712,7 @@ class MacPortsDistribution(CachedDistribution):
 			if not extra.startswith("(active)"):
 				continue
 			version = version.lstrip('@')
-			version = re.sub(r"\+.*","",version) # strip variants
+			version = re.sub(r"\+.*", "", version) # strip variants
 			zi_arch = '*'
 			clean_version = try_cleanup_distro_version(version)
 			if clean_version:
@@ -754,11 +737,49 @@ class MacPortsDistribution(CachedDistribution):
 			impl = factory('package:macports:%s:%s:%s' % (package, version, machine))
 			impl.version = model.parse_version(version)
 			if machine != '*':
-			    impl.machine = machine
+				impl.machine = machine
 
 	def get_score(self, disto_name):
 		return int(disto_name == 'MacPorts')
 
+class CygwinDistribution(CachedDistribution):
+	"""A Cygwin-based distribution."""
+
+	cache_leaf = 'cygcheck-status.cache'
+
+	def generate_cache(self):
+		cache = []
+
+		zi_arch = canonical_machine(arch)
+		for line in os.popen("cygcheck -c -d"):
+			if line == "Cygwin Package Information\r\n":
+				continue
+			if line == "\n":
+				continue
+			package, version = line.split()
+			if package == "Package" and version == "Version":
+				continue
+			clean_version = try_cleanup_distro_version(version)
+			if clean_version:
+				cache.append('%s\t%s\t%s' % (package, clean_version, zi_arch))
+			else:
+				warn(_("Can't parse distribution version '%(version)s' for package '%(package)s'"), {'version': version, 'package': package})
+
+		self._write_cache(cache)
+
+	def get_package_info(self, package, factory):
+		# Add installed versions...
+		versions = self.versions.get(package, [])
+
+		for version, machine in versions:
+			impl = factory('package:cygwin:%s:%s:%s' % (package, version, machine))
+			impl.version = model.parse_version(version)
+			if machine != '*':
+				impl.machine = machine
+
+	def get_score(self, disto_name):
+		return int(disto_name == 'Cygwin')
+
 
 _host_distribution = None
 def get_host_distribution():
@@ -768,16 +789,15 @@ def get_host_distribution():
 	global _host_distribution
 	if not _host_distribution:
 		dpkg_db_status = '/var/lib/dpkg/status'
-		pkgcache = '/var/cache/apt/pkgcache.bin'
 		rpm_db_packages = '/var/lib/rpm/Packages'
 		_slack_db = '/var/log/packages'
 		_arch_db = '/var/lib/pacman'
 		_pkg_db = '/var/db/pkg'
 		_macports_db = '/opt/local/var/macports/registry/registry.db'
+		_cygwin_log = '/var/log/setup.log'
 
 		if sys.prefix == "/sw":
 			dpkg_db_status = os.path.join(sys.prefix, dpkg_db_status)
-			pkgcache = os.path.join(sys.prefix, pkgcache)
 			rpm_db_packages = os.path.join(sys.prefix, rpm_db_packages)
 
 		if os.name == "nt":
@@ -790,9 +810,11 @@ def get_host_distribution():
 		elif os.path.isfile(_macports_db) \
 			and sys.prefix.startswith("/opt/local"):
 				_host_distribution = MacPortsDistribution(_macports_db)
+		elif os.path.isfile(_cygwin_log) and sys.platform == "cygwin":
+			_host_distribution = CygwinDistribution(_cygwin_log)
 		elif os.access(dpkg_db_status, os.R_OK) \
 			and os.path.getsize(dpkg_db_status) > 0:
-			_host_distribution = DebianDistribution(dpkg_db_status, pkgcache)
+			_host_distribution = DebianDistribution(dpkg_db_status)
 		elif os.path.isfile(rpm_db_packages):
 			_host_distribution = RPMDistribution(rpm_db_packages)
 		elif os.path.isdir(_slack_db):
