@@ -30,10 +30,9 @@ creating new cache objects.
 
 import os, sys, time
 from logging import debug, info, warn
-from cStringIO import StringIO
 
 from zeroinstall import _
-from zeroinstall.support import basedir
+from zeroinstall.support import basedir, portable_rename
 from zeroinstall.injector import reader, model
 from zeroinstall.injector.namespaces import config_site, config_prog
 from zeroinstall.injector.model import Interface, escape, unescape
@@ -255,18 +254,6 @@ class IfaceCache(object):
 		debug(_("Updating '%(interface)s' from network; modified at %(time)s") %
 			{'interface': feed_url, 'time': _pretty_time(modified_time)})
 
-		if '\n<!-- Base64 Signature' not in new_xml:
-			# Only do this for old-style feeds without
-			# signatures Otherwise, we can get the time from the
-			# signature, and adding this attribute just makes the
-			# signature invalid.
-			from xml.dom import minidom
-			doc = minidom.parseString(new_xml)
-			doc.documentElement.setAttribute('last-modified', str(modified_time))
-			new_xml = StringIO()
-			doc.writexml(new_xml)
-			new_xml = new_xml.getvalue()
-
 		self._import_new_feed(feed_url, new_xml, modified_time)
 
 		feed = self.get_feed(feed_url)
@@ -302,30 +289,34 @@ class IfaceCache(object):
 
 		# Do we need to write this temporary file now?
 		stream = open(cached + '.new', 'w')
-		stream.write(new_xml)
-		stream.close()
-		os.utime(cached + '.new', (modified_time, modified_time))
-		new_mtime = reader.check_readable(feed_url, cached + '.new')
-		assert new_mtime == modified_time
+		try:
+			stream.write(new_xml)
+			stream.close()
+			os.utime(cached + '.new', (modified_time, modified_time))
+			new_mtime = reader.check_readable(feed_url, cached + '.new')
+			assert new_mtime == modified_time
 
-		old_modified = self._get_signature_date(feed_url) or old_modified
+			old_modified = self._get_signature_date(feed_url) or old_modified
 
-		if old_modified:
-			if new_mtime < old_modified:
-				os.unlink(cached + '.new')
-				raise ReplayAttack(_("New feed's modification time is "
-					"before old version!\nInterface: %(iface)s\nOld time: %(old_time)s\nNew time: %(new_time)s\n"
-					"Refusing update.")
-					% {'iface': feed_url, 'old_time': _pretty_time(old_modified), 'new_time': _pretty_time(new_mtime)})
-			if new_mtime == old_modified:
-				# You used to have to update the modification time manually.
-				# Now it comes from the signature, this check isn't useful
-				# and often causes problems when the stored format changes
-				# (e.g., when we stopped writing last-modified attributes)
-				pass
-				#raise SafeException("Interface has changed, but modification time "
-				#		    "hasn't! Refusing update.")
-		os.rename(cached + '.new', cached)
+			if old_modified:
+				if new_mtime < old_modified:
+					raise ReplayAttack(_("New feed's modification time is "
+						"before old version!\nInterface: %(iface)s\nOld time: %(old_time)s\nNew time: %(new_time)s\n"
+						"Refusing update.")
+						% {'iface': feed_url, 'old_time': _pretty_time(old_modified), 'new_time': _pretty_time(new_mtime)})
+				if new_mtime == old_modified:
+					# You used to have to update the modification time manually.
+					# Now it comes from the signature, this check isn't useful
+					# and often causes problems when the stored format changes
+					# (e.g., when we stopped writing last-modified attributes)
+					pass
+					#raise SafeException("Interface has changed, but modification time "
+					#		    "hasn't! Refusing update.")
+		except:
+			os.unlink(cached + '.new')
+			raise
+
+		portable_rename(cached + '.new', cached)
 		debug(_("Saved as %s") % cached)
 
 		self.get_feed(feed_url, force = True)
@@ -518,26 +509,33 @@ class IfaceCache(object):
 		debug(_("Feed targets: %s"), feed_targets)
 		return [self.get_interface(uri) for uri in feed_targets]
 
-	def is_stale(self, feed, freshness_threshold):
+	def is_stale(self, feed_url, freshness_threshold):
 		"""Check whether feed needs updating, based on the configured L{config.Config.freshness}.
 		None is considered to be stale.
 		If we already tried to update the feed within FAILED_CHECK_DELAY, returns false.
 		@return: True if feed should be updated
 		@since: 0.53"""
-		if feed is None:
-			return True
-		if os.path.isabs(feed.url):
-			return False		# Local feeds are never stale
-		if feed.last_modified is None:
-			return True		# Don't even have it yet
+		if isinstance(feed_url, model.ZeroInstallFeed):
+			feed_url = feed_url.url		# old API
+		elif feed_url is None:
+			return True			# old API
+
 		now = time.time()
-		staleness = now - (feed.last_checked or 0)
-		debug(_("Staleness for %(feed)s is %(staleness).2f hours"), {'feed': feed, 'staleness': staleness / 3600.0})
 
-		if freshness_threshold <= 0 or staleness < freshness_threshold:
-			return False		# Fresh enough for us
+		feed = self.get_feed(feed_url)
+		if feed is not None:
+			if feed.local_path is not None:
+				return False		# Local feeds are never stale
 
-		last_check_attempt = self.get_last_check_attempt(feed.url)
+			if feed.last_modified is not None:
+				staleness = now - (feed.last_checked or 0)
+				debug(_("Staleness for %(feed)s is %(staleness).2f hours"), {'feed': feed, 'staleness': staleness / 3600.0})
+
+				if freshness_threshold <= 0 or staleness < freshness_threshold:
+					return False		# Fresh enough for us
+		# else we've never had it
+
+		last_check_attempt = self.get_last_check_attempt(feed_url)
 		if last_check_attempt and last_check_attempt > now - FAILED_CHECK_DELAY:
 			debug(_("Stale, but tried to check recently (%s) so not rechecking now."), time.ctime(last_check_attempt))
 			return False
@@ -546,6 +544,7 @@ class IfaceCache(object):
 
 	def usable_feeds(self, iface, arch):
 		"""Generator for C{iface.feeds} that are valid for this architecture.
+		@type iface: L{model.Interface}
 		@rtype: generator
 		@see: L{arch}
 		@since: 0.53"""
