@@ -151,7 +151,7 @@ def process_depends(item, local_feed_dir):
 	dep_iface = item.getAttribute('interface')
 	if not dep_iface:
 		raise InvalidInterface(_("Missing 'interface' on <%s>") % item.name)
-	if dep_iface.startswith('./'):
+	if dep_iface.startswith('.'):
 		if local_feed_dir:
 			dep_iface = os.path.abspath(os.path.join(local_feed_dir, dep_iface))
 			# (updates the element too, in case we write it out again)
@@ -275,7 +275,8 @@ class EnvironmentBinding(Binding):
 
 	
 	def __str__(self):
-		return _("<environ %(name)s %(mode)s %(insert)s %(value)s>") % {'name': self.name,'mode': self.mode, 'insert': self.insert, 'value': self.value}
+		return _("<environ %(name)s %(mode)s %(insert)s %(value)s>") % \
+			{'name': self.name, 'mode': self.mode, 'insert': self.insert, 'value': self.value}
 
 	__repr__ = __str__
 	
@@ -378,14 +379,15 @@ class OverlayBinding(Binding):
 class Feed(object):
 	"""An interface's feeds are other interfaces whose implementations can also be
 	used as implementations of this interface."""
-	__slots__ = ['uri', 'os', 'machine', 'user_override', 'langs']
-	def __init__(self, uri, arch, user_override, langs = None):
+	__slots__ = ['uri', 'os', 'machine', 'user_override', 'langs', 'site_package']
+	def __init__(self, uri, arch, user_override, langs = None, site_package = False):
 		self.uri = uri
 		# This indicates whether the feed comes from the user's overrides
 		# file. If true, writer.py will write it when saving.
 		self.user_override = user_override
 		self.os, self.machine = _split_arch(arch)
 		self.langs = langs
+		self.site_package = site_package
 	
 	def __str__(self):
 		return "<Feed from %s>" % self.uri
@@ -681,14 +683,26 @@ class Implementation(object):
 class DistributionImplementation(Implementation):
 	"""An implementation provided by the distribution. Information such as the version
 	comes from the package manager.
+	@ivar package_implementation: the <package-implementation> element that generated this impl (since 1.7)
+	@type package_implementation: L{qdom.Element}
 	@since: 0.28"""
-	__slots__ = ['distro', 'installed']
+	__slots__ = ['distro', 'installed', 'package_implementation']
 
-	def __init__(self, feed, id, distro):
+	def __init__(self, feed, id, distro, package_implementation = None):
 		assert id.startswith('package:')
 		Implementation.__init__(self, feed, id)
 		self.distro = distro
 		self.installed = False
+		self.package_implementation = package_implementation
+
+		if package_implementation:
+			for child in package_implementation.childNodes:
+				if child.uri != XMLNS_IFACE: continue
+				if child.name == 'command':
+					command_name = child.attrs.get('name', None)
+					if not command_name:
+						raise InvalidInterface('Missing name for <command>')
+					self.commands[command_name] = Command(child, local_dir = None)
 
 	@property
 	def requires_root_install(self):
@@ -841,6 +855,8 @@ class ZeroInstallFeed(object):
 	@type descriptions: {str: str}
 	@ivar last_modified: timestamp on signature
 	@ivar last_checked: time feed was last successfully downloaded and updated
+	@ivar local_path: the path of this local feed, or None if remote (since 1.7)
+	@type local_path: str | None
 	@ivar feeds: list of <feed> elements in this feed
 	@type feeds: [L{Feed}]
 	@ivar feed_for: interfaces for which this could be a feed
@@ -849,13 +865,14 @@ class ZeroInstallFeed(object):
 	"""
 	# _main is deprecated
 	__slots__ = ['url', 'implementations', 'name', 'descriptions', 'first_description', 'summaries', 'first_summary', '_package_implementations',
-		     'last_checked', 'last_modified', 'feeds', 'feed_for', 'metadata']
+		     'last_checked', 'last_modified', 'feeds', 'feed_for', 'metadata', 'local_path']
 
 	def __init__(self, feed_element, local_path = None, distro = None):
 		"""Create a feed object from a DOM.
 		@param feed_element: the root element of a feed file
 		@type feed_element: L{qdom.Element}
 		@param local_path: the pathname of this local feed, or None for remote feeds"""
+		self.local_path = local_path
 		self.implementations = {}
 		self.name = None
 		self.summaries = {}	# { lang: str }
@@ -886,6 +903,7 @@ class ZeroInstallFeed(object):
 			self.url = local_path
 			local_dir = os.path.dirname(local_path)
 		else:
+			assert local_path is None
 			self.url = feed_element.getAttribute('uri')
 			if not self.url:
 				raise InvalidInterface(_("<interface> uri attribute missing"))
@@ -1154,6 +1172,18 @@ class ZeroInstallFeed(object):
 	def description(self):
 		return _best_language_match(self.descriptions) or self.first_description
 
+	def get_replaced_by(self):
+		"""Return the URI of the interface that replaced the one with the URI of this feed's URL.
+		This is the value of the feed's <replaced-by interface'...'/> element.
+		@return: the new URI, or None if it hasn't been replaced
+		@since: 1.7"""
+		for child in self.metadata:
+			if child.uri == XMLNS_IFACE and child.name == 'replaced-by':
+				new_uri = child.getAttribute('interface')
+				if new_uri and (new_uri.startswith('http:') or new_uri.startswith('https:') or self.local_path):
+					return new_uri
+		return None
+
 class DummyFeed(object):
 	"""Temporary class used during API transition."""
 	last_modified = None
