@@ -42,15 +42,18 @@ class PackageKit(object):
 	@property
 	def pk(self):
 		if self._pk is False:
-			try:
-				self._pk = dbus.Interface(dbus.SystemBus().get_object(
-							'org.freedesktop.PackageKit',
-							'/org/freedesktop/PackageKit', False),
-						'org.freedesktop.PackageKit')
-				_logger_pk.info(_('PackageKit dbus service found'))
-			except Exception as ex:
-				_logger_pk.info(_('PackageKit dbus service not found: %s'), ex)
+			if dbus is None:
 				self._pk = None
+			else:
+				try:
+					self._pk = dbus.Interface(dbus.SystemBus().get_object(
+								'org.freedesktop.PackageKit',
+								'/org/freedesktop/PackageKit', False),
+							'org.freedesktop.PackageKit')
+					_logger_pk.info(_('PackageKit dbus service found'))
+				except Exception as ex:
+					_logger_pk.info(_('PackageKit dbus service not found: %s'), ex)
+					self._pk = None
 		return self._pk
 
 	def get_candidates(self, package_name, factory, prefix):
@@ -196,7 +199,7 @@ class PackageKitDownload:
 			self.downloaded.trigger(exception = (ex, None))
 
 		def installed_cb(sender):
-			self._impl.installed = True;
+			self._impl.installed = True
 			self.status = download.download_complete
 			self.downloaded.trigger()
 
@@ -204,8 +207,8 @@ class PackageKitDownload:
 			package_name = self.packagekit_id
 			self._transaction = _PackageKitTransaction(self.pk, installed_cb, error_cb)
 			self._transaction.compat_call([
-					('InstallPackages', [package_name]),
 					('InstallPackages', False, [package_name]),
+					('InstallPackages', [package_name]),
 					])
 
 		_auth_wrapper(install_packages)
@@ -289,8 +292,8 @@ class _PackageKitTransaction(object):
 		defaultlocale = locale.getdefaultlocale()[0]
 		if defaultlocale is not None:
 			self.compat_call([
-					('SetLocale', defaultlocale),
 					('SetHints', ['locale=%s' % defaultlocale]),
+					('SetLocale', defaultlocale),
 					])
 
 	def getPercentage(self):
@@ -305,6 +308,8 @@ class _PackageKitTransaction(object):
 		except:
 			return default
 
+	# note: Ubuntu's aptdaemon implementation of PackageKit crashes if passed the wrong
+	# arguments (rather than returning InvalidArgs), so always try its API first.
 	def compat_call(self, calls):
 		for call in calls:
 			method = call[0]
@@ -313,8 +318,9 @@ class _PackageKitTransaction(object):
 				dbus_method = self.proxy.get_dbus_method(method)
 				return dbus_method(*args)
 			except dbus.exceptions.DBusException as e:
-				if e.get_dbus_name() != \
-						'org.freedesktop.DBus.Error.UnknownMethod':
+				if e.get_dbus_name() not in (
+				   'org.freedesktop.DBus.Error.UnknownMethod',
+				   'org.freedesktop.DBus.Error.InvalidArgs'):
 					raise
 		raise Exception('Cannot call %r DBus method' % calls)
 
@@ -335,19 +341,23 @@ class _PackageKitTransaction(object):
 		self.error_details = details
 
 	def __package_cb(self, status, id, summary):
-		from zeroinstall.injector import distro
+		try:
+			from zeroinstall.injector import distro
 
-		package_name, version, arch, repo_ = id.split(';')
-		clean_version = distro.try_cleanup_distro_version(version)
-		if not clean_version:
-			_logger_pk.warn(_("Can't parse distribution version '%(version)s' for package '%(package)s'"), {'version': version, 'package': package_name})
-		clean_arch = distro.canonical_machine(arch)
-		package = {'version': clean_version,
-			   'name': package_name,
-			   'arch': clean_arch,
-			   'installed': (status == 'installed')}
-		_logger_pk.debug(_('Package: %s %r'), id, package)
-		self.package[str(id)] = package
+			package_name, version, arch, repo_ = id.split(';')
+			clean_version = distro.try_cleanup_distro_version(version)
+			if not clean_version:
+				_logger_pk.info(_("Can't parse distribution version '%(version)s' for package '%(package)s'"), {'version': version, 'package': package_name})
+				return
+			clean_arch = distro.canonical_machine(arch)
+			package = {'version': clean_version,
+				   'name': package_name,
+				   'arch': clean_arch,
+				   'installed': (status == 'installed')}
+			_logger_pk.debug(_('Package: %s %r'), id, package)
+			self.package[str(id)] = package
+		except Exception as ex:
+			_logger_pk.warn("__package_cb(%s, %s, %s): %s", status, id, summary, ex)
 
 	def __details_cb(self, id, licence, group, detail, url, size):
 		details = {'licence': str(licence),
