@@ -30,24 +30,7 @@ def do_env_binding(binding, path):
 					os.environ.get(binding.name, None))
 	info("%s=%s", binding.name, os.environ[binding.name])
 
-def execute(policy, prog_args, dry_run = False, main = None, wrapper = None):
-	"""Execute program. On success, doesn't return. On failure, raises an Exception.
-	Returns normally only for a successful dry run.
-	@param policy: a policy with the selected versions
-	@type policy: L{policy.Policy}
-	@param prog_args: arguments to pass to the program
-	@type prog_args: [str]
-	@param dry_run: if True, just print a message about what would have happened
-	@type dry_run: bool
-	@param main: the name of the binary to run, or None to use the default
-	@type main: str
-	@param wrapper: a command to use to actually run the binary, or None to run the binary directly
-	@type wrapper: str
-	@precondition: C{policy.ready and policy.get_uncached_implementations() == []}
-	"""
-	execute_selections(policy.solver.selections, prog_args, dry_run, main, wrapper)
-
-def test_selections(selections, prog_args, dry_run, main, wrapper = None):
+def test_selections(selections, prog_args, dry_run, main):
 	"""Run the program in a child process, collecting stdout and stderr.
 	@return: the output produced by the process
 	@since: 0.27
@@ -115,12 +98,12 @@ class Setup(object):
 		@return: the argument list
 		@rtype: [str]"""
 
-		assert command_name
+		assert command_name or user_command
 
 		prog_args = []
 		sels = self.selections.selections
 
-		while command_name:
+		while command_name or user_command:
 			command_sel = sels[command_iface]
 
 			if user_command is None:
@@ -150,7 +133,7 @@ class Setup(object):
 					if command_path.startswith('/'):
 						raise SafeException(_("Command path must be relative, but '%s' starts with '/'!") %
 									command_path)
-					prog_path = os.path.join(self._get_implementation_path(command_sel), command_path)
+					prog_path = os.path.join(command_sel.get_path(self.stores), command_path)
 
 				assert prog_path is not None
 
@@ -174,10 +157,6 @@ class Setup(object):
 
 		return prog_args
 
-	def _get_implementation_path(self, impl):
-		if impl.id.startswith('package:'): return None
-		return impl.local_path or self.stores.lookup_any(impl.digests)
-
 	def prepare_env(self):
 		"""Do all the environment bindings in the selections (setting os.environ)."""
 		self._exec_bindings = []
@@ -220,7 +199,11 @@ class Setup(object):
 		@type iface: L{model.Interface}
 		"""
 		if isinstance(binding, EnvironmentBinding):
-			do_env_binding(binding, self._get_implementation_path(impl))
+			if impl.id.startswith('package:'):
+				path = None		# (but still do the binding, e.g. for values)
+			else:
+				path = impl.get_path(self.stores)
+			do_env_binding(binding, path)
 		elif isinstance(binding, ExecutableBinding):
 			if isinstance(iface, Dependency):
 				import warnings
@@ -333,6 +316,11 @@ def execute_selections(selections, prog_args, dry_run = False, main = None, wrap
 		sys.stdout.flush()
 		sys.stderr.flush()
 		try:
-			os.execv(prog_args[0], prog_args)
+			env = os.environ.copy()
+			for x in ['0install-runenv-ZEROINSTALL_GPG', 'ZEROINSTALL_GPG']:
+				if x in env:
+					del env[x]
+
+			os.execve(prog_args[0], prog_args, env)
 		except OSError as ex:
 			raise SafeException(_("Failed to run '%(program_path)s': %(exception)s") % {'program_path': prog_args[0], 'exception': str(ex)})
