@@ -9,7 +9,7 @@ Load and save a set of chosen implementations.
 import os
 from zeroinstall import _
 from zeroinstall.injector import model
-from zeroinstall.injector.policy import Policy, get_deprecated_singleton_config
+from zeroinstall.injector.policy import get_deprecated_singleton_config
 from zeroinstall.injector.model import process_binding, process_depends, binding_names, Command
 from zeroinstall.injector.namespaces import XMLNS_IFACE
 from zeroinstall.injector.qdom import Element, Prefixes
@@ -53,6 +53,27 @@ class Selection(object):
 		path = stores.lookup_maybe(self.digests)
 		return path is not None
 
+	def get_path(self, stores, missing_ok = False):
+		"""Return the root directory of this implementation.
+		For local implementations, this is L{local_path}.
+		For cached implementations, this is the directory in the cache.
+		@param stores: stores to search
+		@type stores: L{zerostore.Stores}
+		@param missing_ok: return None for uncached implementations
+		@type missing_ok: bool
+		@return: the path of the directory
+		@rtype: str | None
+		@since: 1.8"""
+		if self.local_path is not None:
+			return self.local_path
+		if not self.digests:
+			# (for now, we assume this is always an error, even for missing_ok)
+			raise model.SafeException("No digests for {feed} {version}".format(feed = self.feed, version = self.version))
+		if missing_ok:
+			return stores.lookup_maybe(self.digests)
+		else:
+			return stores.lookup_any(self.digests)
+
 class ImplSelection(Selection):
 	"""A Selection created from an Implementation"""
 
@@ -141,23 +162,11 @@ class Selections(object):
 		if source is None:
 			# (Solver will fill everything in)
 			pass
-		elif isinstance(source, Policy):
-			import warnings
-			warnings.warn("Use policy.solver.selections instead", DeprecationWarning, 2)
-			self._init_from_policy(source)
 		elif isinstance(source, Element):
 			self._init_from_qdom(source)
 		else:
 			raise Exception(_("Source not a qdom.Element!"))
 
-	def _init_from_policy(self, policy):
-		"""Set the selections from a policy.
-		@deprecated: use Solver.selections instead
-		@param policy: the policy giving the selected implementations."""
-		self.interface = policy.root
-		self.selections = policy.solver.selections.selections
-		self.commands = policy.solver.selections.commands
-
 	def _init_from_qdom(self, root):
 		"""Parse and load a selections document.
 		@param root: a saved set of selections."""
@@ -312,7 +321,7 @@ class Selections(object):
 			root.setAttributeNS(XMLNS_NAMESPACE, 'xmlns:' + prefix, ns)
 
 		return doc
-	
+
 	def __repr__(self):
 		return "Selections for " + self.interface
 
@@ -325,8 +334,6 @@ class Selections(object):
 		@param config: used to get iface_cache, stores and fetcher
 		@param include_packages: also try to install native packages (since 1.5)
 		@return: a L{tasks.Blocker} or None"""
-		from zeroinstall.zerostore import NotStored
-
 		if _old:
 			config = get_deprecated_singleton_config()
 
@@ -344,10 +351,7 @@ class Selections(object):
 			elif sel.local_path:
 				return False
 			else:
-				try:
-					stores.lookup_any(sel.digests)
-				except NotStored:
-					return True
+				return sel.get_path(stores, missing_ok = True) is None
 
 		needed_downloads = list(filter(needs_download, self.selections.values()))
 		if not needed_downloads:
