@@ -10,8 +10,7 @@ import locale
 from logging import debug, warn, info
 
 from zeroinstall.injector.reader import MissingLocalFeed
-from zeroinstall.injector.arch import machine_groups
-from zeroinstall.injector import model, sat, selections
+from zeroinstall.injector import model, sat, selections, arch
 
 class CommandInfo:
 	def __init__(self, name, command, impl, arch):
@@ -55,6 +54,15 @@ class _DummyImpl(object):
 	def get_name(self):
 		return "dummy"
 
+class _ForceImpl(model.Restriction):
+	"""Used by L{SATSolver.justify_decision}."""
+
+	def __init__(self, impl_id):
+		self.impl_id = impl_id
+
+	def meets_restriction(self, impl):
+		return impl.id == self.impl_id
+
 class Solver(object):
 	"""Chooses a set of implementations to satisfy the requirements of a program and its user.
 	Typical use:
@@ -80,6 +88,14 @@ class Solver(object):
 		self.record_details = False
 		self.ready = False
 
+	def solve_for(self, requirements):
+		"""Solve for given requirements.
+		@param requirements: the interface, architecture and command to solve for
+		@type requirements: L{requirements.Requirements}
+		@postcondition: self.ready, self.selections and self.feeds_used are updated
+		@since: 1.8"""
+		return self.solve(requirements.interface_uri, self.get_arch_for(requirements), requirements.command)
+
 	def solve(self, root_interface, root_arch, command_name = 'run'):
 		"""Get the best implementation of root_interface and all of its dependencies.
 		@param root_interface: the URI of the program to be solved
@@ -91,6 +107,33 @@ class Solver(object):
 		@postcondition: self.ready, self.selections and self.feeds_used are updated"""
 		raise NotImplementedError("Abstract")
 
+	def get_arch_for(self, requirements, interface = None):
+		"""Return the Architecture we would use when solving for this interface.
+		Normally, this architecture is constructed from the OS and CPU type in the requirements,
+		using the host platform's settings if these are not given.
+		If interface is the root, then we wrap this in a SourceArchitecture if looking
+		for source code and (for backwards compatibility) we enable use="testing" dependencies
+		if the command is "test".
+		@param requirements: the overall requirements for the solve
+		@type requirements: L{requirements.Requirements}
+		@param interface: the interface of interest
+		@type interface: L{model.Interface}
+		@return: the architecture that would be used
+		@rtype: L{architecture.Architecture}
+		@since: 1.9"""
+		root_arch = arch.get_architecture(requirements.os, requirements.cpu)
+		if interface is None or interface.uri == requirements.interface_uri:
+			if requirements.source:
+				root_arch = arch.SourceArchitecture(root_arch)
+			if requirements.command == 'test':
+				# This is for old feeds that have use='testing' instead of the newer
+				# 'test' command for giving test-only dependencies.
+				root_arch = arch.Architecture(root_arch.os_ranks, root_arch.machine_ranks)
+				root_arch.use = frozenset([None, "testing"])
+			return root_arch
+		# Assume we use the same arch for all descendants
+		return root_arch.child_arch
+
 class SATSolver(Solver):
 	"""Converts the problem to a set of pseudo-boolean constraints and uses a PB solver to solve them.
 	@ivar langs: the preferred languages (e.g. ["es_ES", "en"]). Initialised to the current locale.
@@ -157,6 +200,7 @@ class SATSolver(Solver):
 		else:
 			stability_limited = stability
 
+		# Note: this list must match _ranking_component_reason above
 		return [
 			# Languages we understand come first
 			max(my_langs.get(l.split('-')[0], -1) for l in impl_langs),
@@ -239,6 +283,7 @@ class SATSolver(Solver):
 
 		ifaces_processed = set()
 
+		machine_groups = arch.machine_groups
 		impls_for_machine_group = {0 : []}		# Machine group (e.g. "64") to [impl] in that group
 		for machine_group in machine_groups.values():
 			impls_for_machine_group[machine_group] = []
@@ -327,6 +372,8 @@ class SATSolver(Solver):
 			if impl.machine not in arch.machine_ranks:
 				if impl.machine == 'src':
 					return _("Source code")
+				elif 'src' in arch.machine_ranks:
+					return _("Not source code")
 				return _("Unsupported machine type")
 			return None
 
@@ -407,7 +454,6 @@ class SATSolver(Solver):
 			if self.record_details:
 				self.details[iface] = [(impl, get_unusable_reason(impl, my_extra_restrictions, arch)) for impl in impls]
 
-			rank = 1
 			var_names = []
 			for impl in impls:
 				if is_unusable(impl, my_extra_restrictions, arch):
@@ -418,7 +464,6 @@ class SATSolver(Solver):
 				assert impl not in impl_to_var
 				v = problem.add_variable(ImplInfo(iface, impl, arch))
 				impl_to_var[impl] = v
-				rank += 1
 				var_names.append(v)
 
 				if impl.machine and impl.machine != 'src':
@@ -570,7 +615,13 @@ class SATSolver(Solver):
 					return	# Break cycles
 				seen.add(uri)
 
-				group = group_clause_for[uri]
+				group = group_clause_for.get(uri, None)
+
+				if group is None:
+					# (can be None if the selected impl has an optional dependency on 
+					# a feed with no implementations)
+					return
+
 				#print "Group for %s = %s" % (uri, group)
 				lit = group.current
 				if lit is None:
@@ -629,7 +680,7 @@ class SATSolver(Solver):
 
 			commands_needed = []
 
-			# Popular sels with the selected implementations.
+			# Populate sels with the selected implementations.
 			# Also, note down all the commands we need.
 			for uri, group in group_clause_for.iteritems():
 				if group.current is not None:
@@ -691,4 +742,104 @@ class SATSolver(Solver):
 				'\n'.join(["- %s -> %s" % (iface, self.selections[iface])
 					   for iface  in self.selections]))
 
+	def justify_decision(self, requirements, iface, impl):
+		"""Run a solve with impl_id forced to be selected, and explain why it wasn't (or was)
+		selected in the normal case."""
+		assert isinstance(iface, model.Interface), iface
+
+		restrictions = self.extra_restrictions.copy()
+		restrictions[iface] = restrictions.get(iface, []) + [_ForceImpl(impl.id)]
+		s = SATSolver(self.config, restrictions)
+		s.record_details = True
+		s.solve_for(requirements)
+
+		wanted = "{iface} {version}".format(iface = iface.get_name(), version = impl.get_version())
+
+		# Could a selection involving impl even be valid?
+		if not s.ready or iface.uri not in s.selections.selections:
+			reasons = s.details.get(iface, [])
+			for (rid, rstr) in reasons:
+				if rid.id == impl.id and rstr is not None:
+					return _("{wanted} cannot be used (regardless of other components): {reason}").format(
+							wanted = wanted,
+							reason = rstr)
+
+			if not s.ready:
+				return _("There is no possible selection using {wanted}.\n{reason}").format(
+					wanted = wanted,
+					reason = s.get_failure_reason())
+
+		actual_selection = self.selections.get(iface, None)
+		if actual_selection is not None:
+			# Was impl actually selected anyway?
+			if actual_selection.id == impl.id:
+				return _("{wanted} was selected as the preferred version.").format(wanted = wanted)
+
+			# Was impl ranked below the selected version?
+			iface_arch = arch.get_architecture(requirements.os, requirements.cpu)
+			if requirements.source and iface.uri == requirements.interface_uri:
+				iface_arch = arch.SourceArchitecture(iface_arch)
+			wanted_rating = self.get_rating(iface, impl, arch)
+			selected_rating = self.get_rating(iface, actual_selection, arch)
+
+			if wanted_rating < selected_rating:
+				_ranking_component_reason = [
+					_("natural languages we understand are preferred"),
+					_("preferred versions come first"),
+					_("locally-available versions are preferred when network use is limited"),
+					_("packages that don't require admin access to install are preferred"),
+					_("more stable versions preferred"),
+					_("newer versions are preferred"),
+					_("native packages are preferred"),
+					_("newer versions are preferred"),
+					_("better OS match"),
+					_("better CPU match"),
+					_("better locale match"),
+					_("is locally available"),
+					_("better ID (tie-breaker)"),
+				]
+
+				actual = actual_selection.get_version()
+				if impl.get_version() == actual:
+					def detail(i):
+						if len(i.id) < 18:
+							return " (" + i.id + ")"
+						else:
+							return " (" + i.id[:16] + "...)"
+
+					wanted += detail(impl)
+					actual += detail(actual_selection)
+
+				for i in range(len(wanted_rating)):
+					if wanted_rating[i] < selected_rating[i]:
+						return _("{wanted} is ranked lower than {actual}: {why}").format(
+								wanted = wanted,
+								actual = actual,
+								why = _ranking_component_reason[i])
+
+		used_impl = iface.uri in s.selections.selections
+
+		# Impl is selectable and ranked higher than the selected version. Selecting it would cause
+		# a problem elsewhere.
+		changes = []
+		for old_iface, old_sel in self.selections.selections.iteritems():
+			if old_iface == iface.uri and used_impl: continue
+			new_sel = s.selections.selections.get(old_iface, None)
+			if new_sel is None:
+				changes.append(_("{interface}: no longer used").format(interface = old_iface))
+			elif old_sel.version != new_sel.version:
+				changes.append(_("{interface}: {old} to {new}").format(interface = old_iface, old = old_sel.version, new = new_sel.version))
+			elif old_sel.id != new_sel.id:
+				changes.append(_("{interface}: {old} to {new}").format(interface = old_iface, old = old_sel.id, new = new_sel.id))
+
+		if changes:
+			changes_text = '\n\n' + _('The changes would be:') + '\n\n' + '\n'.join(changes)
+		else:
+			changes_text = ''
+
+		if used_impl:
+			return _("{wanted} is selectable, but using it would produce a less optimal solution overall.").format(wanted = wanted) + changes_text
+		else:
+			return _("If {wanted} were the only option, the best available solution wouldn't use it.").format(wanted = wanted) + changes_text
+
 DefaultSolver = SATSolver
