@@ -14,6 +14,7 @@ from zeroinstall import _, SafeException
 import os
 from logging import info
 
+from zeroinstall import support
 from zeroinstall.support import basedir, tasks
 from .namespaces import config_site, config_prog, XMLNS_TRUST
 
@@ -111,7 +112,7 @@ class TrustDB(object):
 		doc.writexml(tmp, indent = "", addindent = "  ", newl = "\n")
 		tmp.close()
 
-		os.rename(tmpname, os.path.join(d, 'trustdb.xml'))
+		support.portable_rename(tmpname, os.path.join(d, 'trustdb.xml'))
 	
 	def notify(self):
 		"""Call all watcher callbacks.
@@ -138,7 +139,7 @@ class TrustDB(object):
 			trust = basedir.load_first_config(config_site, config_prog, 'trust')
 			if trust:
 				#print "Loading trust from", trust_db
-				for key in file(trust).read().split('\n'):
+				for key in open(trust).read().split('\n'):
 					if key:
 						self.keys[key] = set(['*'])
 			else:
