@@ -47,7 +47,7 @@ def read_bytes(fd, nbytes, null_ok = False):
 					% {'data': repr(data), 'bytes': nbytes})
 		data += got
 		nbytes -= len(got)
-	logging.debug(_("Message received: %s") % repr(data))
+	logging.debug(_("Message received: %r"), data)
 	return data
 
 def pretty_size(size):
@@ -86,7 +86,17 @@ def ro_rmtree(root):
 	shutil.rmtree(root)
 
 def raise_with_traceback(ex, tb):
+	"""Raise an exception in a way that works on Python 2 and Python 3"""
 	if hasattr(ex, 'with_traceback'):
 		raise ex.with_traceback(tb)			# Python 3
 	exec("raise ex, None, tb", {'ex': ex, 'tb': tb})	# Python 2
 	assert 0
+
+def portable_rename(src, dst):
+	"""Rename 'src' to 'dst', which must be on the same filesystem.
+	On POSIX systems, this operation is atomic.
+	On Windows, do the best we can by deleting dst and then renaming.
+	@since: 1.9"""
+	if os.name == "nt" and os.path.exists(dst):
+		os.unlink(dst)
+	os.rename(src, dst)
